@@ -14,8 +14,11 @@
 
 package net.sf.kdgcommons.collections;
 
+import java.lang.reflect.Array;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
@@ -38,17 +41,45 @@ import java.util.regex.Pattern;
 public class CollectionUtil
 {
     /**
-     *  Returns a set containing the passed elements.
+     *  Returns a set (<code>HashSet</code>) containing the passed elements.
      */
-    public static <T> Set<T> asSet(T... elems)
+    public static <T> HashSet<T> asSet(T... elems)
     {
-        Set<T> result = new HashSet<T>();
+        HashSet<T> result = new HashSet<T>();
         for (T elem : elems)
             result.add(elem);
         return result;
     }
 
 
+    /**
+     *  Returns a map (<code>HashMap</code>) built from the passed elements. Elements
+     *  alternate between keys and values, and if given an odd number of elements the
+     *  last will be used as the key for a null value. Elements will be added in the
+     *  order they appear as parameters, so repeating a key will mean that only the
+     *  last value is stored.
+     *  <p>
+     *  Note that the result is parameterized as <code>Object,Object</code>; as there
+     *  is no way to differentiate between keys and values with varargs, we have to
+     *  use the lowest common denominator.
+     *
+     *  @since 1.0.15
+     */
+    public static HashMap<Object,Object> asMap(Object... elems)
+    {
+        HashMap<Object,Object> result = new HashMap<Object,Object>();
+        for (int ii = 0 ; ii < elems.length ; ii += 2)
+        {
+            Object key = elems[ii];
+            Object value = (ii < elems.length - 1)
+                         ? elems[ii + 1]
+                         : null;
+            result.put(key, value);
+        }
+        return result;
+    }
+
+
     /**
      *  Appends an arbitrary number of explicit elements to an existing collection.
      *  Primarily useful when writing testcases.
@@ -87,9 +118,7 @@ public class CollectionUtil
      */
     public static <T> Collection<T> addIf(Collection<T> coll, T value, boolean expr)
     {
-        if (expr)
-            coll.add(value);
-
+        if (expr)  coll.add(value);
         return coll;
     }
 
@@ -106,6 +135,31 @@ public class CollectionUtil
     }
 
 
+    /**
+     *  Stores an entry in a map if the boolean expression is true. Returns the
+     *  map as a convenience for chained invocations.
+     *
+     *  @since 1.0.15
+     */
+    public static <K,V> Map<K,V> putIf(Map<K,V> map, K key, V value, boolean expr)
+    {
+        if (expr) map.put(key, value);
+        return map;
+    }
+
+
+    /**
+     *  Stores an entry in a map if the value is not <code>null</code>. Returns the
+     *  map as a convenience for chained invocations.
+     *
+     *  @since 1.0.15
+     */
+    public static <K,V> Map<K,V> putIfNotNull(Map<K,V> map, K key, V value)
+    {
+        return putIf(map, key, value, value != null);
+    }
+
+
     /**
      *  Adds the specified item to a map if it does not already exist. Returns
      *  either the added item or the existing mapping.
@@ -165,9 +219,12 @@ public class CollectionUtil
 
 
     /**
-     *  Verifies that the passed list contains only elements of the given
-     *  type, and returns it as a parameterized type. Throws if any element
-     *  is a different type.
+     *  Verifies that the passed list contains only elements of the given type,
+     *  and returns it as a parameterized list (does not create a new list).
+     *  <p>
+     *  This function exists to avoid suppressing warnings in application code.
+     *
+     *  @throws ClassCastException if any element is a different type.
      */
     @SuppressWarnings("unchecked")
     public static <T> List<T> cast(List<?> list, Class<T> klass)
@@ -181,9 +238,12 @@ public class CollectionUtil
 
 
     /**
-     *  Verifies that the passed set contains only elements of the given
-     *  type, and returns it as a parameterized type. Throws if any element
-     *  is a different type.
+     *  Verifies that the passed set contains only elements of the given type,
+     *  and returns it as a parameterized set (does not create a new set).
+     *  <p>
+     *  This function exists to avoid suppressing warnings in application code.
+     *
+     *  @throws ClassCastException if any element is a different type.
      */
     @SuppressWarnings("unchecked")
     public static <T> Set<T> cast(Set<?> set, Class<T> klass)
@@ -196,6 +256,28 @@ public class CollectionUtil
     }
 
 
+    /**
+     *  Verifies that the passed set contains only keys and values of the given
+     *  types, and returns it as a parameterized map (does not create a new map).
+     *  <p>
+     *  This function exists to avoid suppressing warnings in application code.
+     *
+     *  @throws ClassCastException if any key/value is a different type.
+     *
+     *  @since 1.0.15
+     */
+    @SuppressWarnings("unchecked")
+    public static <K,V> Map<K,V> cast(Map<?,?> map, Class<K> keyClass, Class<V> valueClass)
+    {
+        for (Map.Entry<?,?> entry : map.entrySet())
+        {
+            keyClass.cast(entry.getKey());
+            valueClass.cast(entry.getValue());
+        }
+        return (Map<K,V>)map;
+    }
+
+
     /**
      *  Resizes the passed list to N entries. Will add the specified object if
      *  the list is smaller than the desired size, discard trailing entries if
@@ -418,6 +500,80 @@ public class CollectionUtil
     }
 
 
+    /**
+     *  Retrieves a value from a nested collection hierarchy by following a sequence
+     *  of keys. Supports arbitrary nesting of maps, arrays, and lists.
+     *
+     *  @param  root    The root object.
+     *  @param  keys    One or more keys. The first key is applied to the root object,
+     *                  the second key is applied to the result of that, and so on.
+     *                  Arrays and lists may only be accessed via numeric keys; maps
+     *                  may be accessed via any type of key.
+     *
+     *  @return The object located via the sequence of keys, <code>null</code> if
+     *          <em>any</em> key along the path does not resolve to an object.
+     *
+     *  @throws IllegalArgumentException if any object found during the traversal is
+     *          not a valid collection type, or if the key is not appropriate for the
+     *          collection.
+     *
+     *  @since 1.0.15
+     */
+    public static Object getVia(Object root, Object... keys)
+    {
+        Object current = root;
+
+        // I iterate by index rather than for-in so that I can construct exceptions
+        for (int ii = 0 ; ii < keys.length ; ii++)
+        {
+            try
+            {
+                Object key = keys[ii];
+                if (current == null)
+                {
+                    return null;
+                }
+                if (current instanceof Map)
+                {
+                    current = ((Map<Object,Object>)current).get(key);
+                }
+                else if (current instanceof List)
+                {
+                    int index = ((Number)key).intValue();
+                    List<Object> list = (List<Object>)current;
+                    current = (index < list.size())
+                            ? list.get(index)
+                            : null;
+                }
+                else if (current.getClass().isArray())
+                {
+                    int index = ((Number)key).intValue();
+                    current = (index < Array.getLength(current))
+                            ? Array.get(current, index)
+                            : null;
+                }
+                else
+                {
+                    List<Object> currentKeys = Arrays.asList(keys).subList(0, ii+1);
+                    throw new IllegalArgumentException(
+                            "attempted to get from " + current.getClass().getName()
+                            + " (path: " + currentKeys + ")");
+                }
+            }
+            catch (ClassCastException ex)
+            {
+                // I believe this is the only way that we can get here ...
+                List<Object> currentKeys = Arrays.asList(keys).subList(0, ii+1);
+                throw new IllegalArgumentException(
+                        "attempted to get from " + current.getClass().getName() + " with non-numeric index"
+                        + " (path: " + currentKeys + ")");
+            }
+        }
+
+        return current;
+    }
+
+
     /**
      *  Applies the specified functor to every element of the given collection, in
      *  its natural iteration order, and returns a list of the results.
